package com.sd.business.service.board.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.board.dto.BzSkuSpecSalesSelectDto;
import com.sd.business.entity.board.dto.DailyBoardSelectDto;
import com.sd.business.entity.board.dto.SalesBoardSelectDto;
import com.sd.business.entity.board.vo.*;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomClassify;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.inventory.po.InventoryFinishedOrder;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.mapper.board.SalesBoardMapper;
import com.sd.business.service.board.SalesBoardService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.inventory.InventoryFinishedOrderService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.sku.SkuSpecService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.Comparator.comparing;

/**
 * <p>
 * 销售看板 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-16
 */
@Service
public class SalesBoardServiceImpl implements SalesBoardService {

    @Resource
    private SalesBoardMapper salesBoardMapper;

    @Autowired
    private OrderService orderService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private InventoryFinishedOrderService inventoryFinishedOrderService;

    @Override
    public Page<SkuSalesRankingVo> getSkuSalesRankingPage(SalesBoardSelectDto dto) {
        IWrapper<SkuSalesRankingVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.ge("oi", OrderInfo::getShippingTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getShippingTime, dto.getEndTime());
        wrapper.groupBy("ss.id");
        wrapper.orderByDesc("salesQuantity");
        return salesBoardMapper.getSkuSalesRankingPage(dto.getPage(), wrapper);
    }

    @Override
    public Page<SkuSalesAmountRankingVo> getSkuSalesAmountRankingPage(SalesBoardSelectDto dto) {
        IWrapper<SkuSalesAmountRankingVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.ge("oi", OrderInfo::getShippingTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getShippingTime, dto.getEndTime());
        wrapper.groupBy("ss.id");
        wrapper.orderByDesc("salesAmount");
        return salesBoardMapper.getSkuSalesAmountRankingPage(dto.getPage(), wrapper);
    }

    @Override
    public Page<BomSalesRankingVo> getBomSalesRankingPage(SalesBoardSelectDto dto) {
        IWrapper<BomSalesRankingVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.ge("oi", OrderInfo::getShippingTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getShippingTime, dto.getEndTime());
        wrapper.groupBy("bs.id");
        wrapper.orderByDesc("salesQuantity");
        return salesBoardMapper.getBomSalesRankingPage(dto.getPage(), wrapper);
    }

    @Override
    public Page<BomSalesAmountRankingVo> getBomSalesAmountRankingPage(SalesBoardSelectDto dto) {
        IWrapper<BomSalesAmountRankingVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.ge("oi", OrderInfo::getShippingTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getShippingTime, dto.getEndTime());
        wrapper.groupBy("bs.id");
        wrapper.orderByDesc("salesAmount");
        return salesBoardMapper.getBomSalesAmountRankingPage(dto.getPage(), wrapper);
    }

    @Override
    public Page<SkuSalesRegionRankingVo> getSkuSalesRegionRankingPage(SalesBoardSelectDto dto) {
        IWrapper<SkuSalesRegionRankingVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.ge("oi", OrderInfo::getShippingTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getShippingTime, dto.getEndTime());
        wrapper.groupBy("oi.city");
        if (Objects.equals(dto.getSortBy(), 1)) {
            wrapper.orderByAsc("salesQuantity");
        } else {
            wrapper.orderByDesc("salesQuantity");
        }
        return salesBoardMapper.getSkuSalesRegionRankingPage(dto.getPage(), wrapper);
    }

    @Override
    public Page<SkuSalesGrowthRateRankingVo> getSkuSalesGrowthRateRankingPage(SalesBoardSelectDto dto) {
        IWrapper<SkuSalesGrowthRateRankingVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        if (Objects.equals(dto.getSortBy(), 1)) {
            wrapper.orderByAsc("monthOnMonthRatio");
        } else {
            wrapper.orderByDesc("monthOnMonthRatio");
        }
        Page<SkuSalesGrowthRateRankingVo> page = salesBoardMapper.getSkuSalesGrowthRateRankingPage(dto.getPage(), wrapper);
        page.getRecords().forEach(item -> {
            item.setMonthOverMonthRatio(item.getMonthOverMonthRatio() + "%");
            item.setMonthOnMonthRatio(item.getMonthOnMonthRatio() + "%");
        });
        return page;
    }

    @Override
    public List<DepartmentBomSalesBoardVo> getDepartmentBomSalesStatisticsList(DailyBoardSelectDto dto) {
        List<OrderInfo> orderList = orderService.list(q -> q
                .ne(OrderInfo::getClassify, OrderClassifyEnum.OUTSOURCE_ORDER.getKey())
                .between(OrderInfo::getShippingTime, DateUtil.beginOfDay(dto.getBeginDate()), DateUtil.endOfDay(dto.getEndDate()))
                .select(BaseIdPo::getId, OrderInfo::getDepartmentId));

        if (orderList.isEmpty()) {
            return Collections.emptyList();
        }

        List<Long> orderIds = orderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<OrderInfo>> orderMap = orderList.stream().collect(Collectors.groupingBy(OrderInfo::getDepartmentId));
        // 获取事业部
        Map<Long, String> departmentMap = departmentService.mapKV(BaseIdPo::getId, Department::getName, q -> q.in(BaseIdPo::getId, orderMap.keySet()));

        // 获取订单sku列表
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q
                .in(OrderSku::getOrderId, orderIds)
                .select(OrderSku::getOrderId, OrderSku::getBomSpecId, OrderSku::getQuantity));
        Map<Long, List<OrderSku>> orderSkuMap = orderSkuList.stream().collect(Collectors.groupingBy(OrderSku::getOrderId));

        // 获取bom列表
        Set<Long> bomSpecIds = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toSet());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);

        List<DepartmentBomSalesBoardVo> list = new ArrayList<>();

        orderMap.forEach((departmentId, orderInfoList) -> {
            DepartmentBomSalesBoardVo bomSalesBoardVo = new DepartmentBomSalesBoardVo();
            bomSalesBoardVo.setDepartmentName(departmentMap.get(departmentId));
            bomSalesBoardVo.setOrderSalesCount(orderInfoList.size());

            // 获取订单bom销售列表
            Map<Long, DailyBoardOutStorageBomVo> bomSpecSalesMap = orderInfoList.stream()
                    .map(item -> orderSkuMap.getOrDefault(item.getId(), Collections.emptyList()))
                    .flatMap(Collection::stream)
                    .filter(item -> {
                        BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                        if (bomSpecBo == null) {
                            return false;
                        }
                        return Objects.equals(bomSpecBo.getClassifyParentId(), 1L);
                    })
                    .collect(Collectors.toMap(
                            OrderSku::getBomSpecId,
                            item -> {
                                DailyBoardOutStorageBomVo bomVo = new DailyBoardOutStorageBomVo();
                                bomVo.setQuantity(item.getQuantity());
                                BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                                bomVo.setBomSpecId(bomSpecBo.getBomSpecId());
                                bomVo.setBomSpecName(bomSpecBo.getBomSpecName());
                                bomVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
                                return bomVo;
                            },
                            (v1, v2) -> {
                                v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                return v1;
                            })
                    );

            bomSalesBoardVo.setBomSpecSalesList(bomSpecSalesMap.values().stream().sorted(comparing(DailyBoardOutStorageBomVo::getQuantity).reversed()).collect(Collectors.toList()));
            BigDecimal bomSpecSalesCount = bomSalesBoardVo.getBomSpecSalesList().stream().map(DailyBoardOutStorageBomVo::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
            bomSalesBoardVo.setBomSpecSalesCount(bomSpecSalesCount);
            list.add(bomSalesBoardVo);
        });

        list.sort(comparing(DepartmentBomSalesBoardVo::getBomSpecSalesCount).reversed());

        // 将所有事业部的bom合并，获取所有bom销售列表
        Map<Long, DailyBoardOutStorageBomVo> sumBomMap = list.stream()
                .map(DepartmentBomSalesBoardVo::getBomSpecSalesList)
                .flatMap(Collection::stream)
                .collect(Collectors.toMap(
                        DailyBoardOutStorageBomVo::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            DailyBoardOutStorageBomVo tempVo = BeanUtil.copyProperties(v1, DailyBoardOutStorageBomVo.class);
                            tempVo.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            return tempVo;
                        }));

        DepartmentBomSalesBoardVo sumBomVo = new DepartmentBomSalesBoardVo();
        sumBomVo.setDepartmentName("");
        sumBomVo.setOrderSalesCount(orderList.size());
        sumBomVo.setBomSpecSalesList(sumBomMap.values().stream().sorted(comparing(DailyBoardOutStorageBomVo::getQuantity).reversed()).collect(Collectors.toList()));
        BigDecimal sumBomSpecSalesCount = sumBomVo.getBomSpecSalesList().stream().map(DailyBoardOutStorageBomVo::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
        sumBomVo.setBomSpecSalesCount(sumBomSpecSalesCount);
        list.add(0, sumBomVo);
        return list;
    }

    @Override
    public List<BzSkuSalesBoardVo> getBzSkuSalesBoard() {
        IWrapper<SkuSalesRankingVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("oi", OrderInfo::getDepartmentId, DepartmentConstant.BZ);
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.ge("oi", OrderInfo::getShippingTime, DateUtil.beginOfDay(DateUtil.offsetDay(new Date(), -30)));
        wrapper.groupBy("ss.id");
        wrapper.orderByDesc("salesQuantity");
        Page<SkuSalesRankingVo> page = salesBoardMapper.getSkuSalesRankingPage(new Page<>(1, 10), wrapper);
        List<SkuSalesRankingVo> records = page.getRecords();
        if (records.isEmpty()) {
            return Collections.emptyList();
        }

        // 获取所有无源可出库的库存
        List<InventoryFinishedOrder> inventoryFinishedOrderList = inventoryFinishedOrderService.list(q -> q
                .eq(InventoryFinishedOrder::getStatus, 1)
                .isNull(InventoryFinishedOrder::getOrderInfoId));
        Map<Long, BigDecimal> inventoryFinishedMap = inventoryFinishedOrderList.stream()
                .collect(Collectors.toMap(
                        InventoryFinishedOrder::getSkuSpecId,
                        InventoryFinishedOrder::getExistingQuantity,
                        BigDecimal::add));

        List<BzSkuSalesBoardVo> salesBoardVoList = records.stream().map(item -> {
            BzSkuSalesBoardVo vo = new BzSkuSalesBoardVo();
            vo.setSkuSpecCode(item.getSkuSpecCode());
            vo.setSkuSpecName(item.getSkuSpecName());
            vo.setDailySales(item.getSalesQuantity().divide(BigDecimal.valueOf(30), 0, RoundingMode.HALF_UP));
            vo.setSafetyStock(vo.getDailySales().multiply(BigDecimal.valueOf(3)));
            vo.setInventoryQuantity(inventoryFinishedMap.getOrDefault(item.getSkuSpecId(), BigDecimal.ZERO));
            return vo;
        }).collect(Collectors.toList());

        return salesBoardVoList;
    }

    @Override
    public Page<BzSkuSpecSalesVo> getBzSkuSpecSalesPage(BzSkuSpecSalesSelectDto dto) {
        IWrapper<SkuSalesRankingVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("oi", OrderInfo::getDepartmentId, DepartmentConstant.BZ);
        wrapper.eq("oi", OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.ge("oi", OrderInfo::getShippingTime, DateUtil.beginOfDay(DateUtil.offsetDay(new Date(), -30)));
        wrapper.groupBy("ss.id");
        wrapper.orderByDesc("salesQuantity");
        Page<SkuSalesRankingVo> page = salesBoardMapper.getSkuSalesRankingPage(dto.getPage(), wrapper);
        List<SkuSalesRankingVo> records = page.getRecords();

        Page<BzSkuSpecSalesVo> result = new Page<>();
        if (records.isEmpty()) {
            return result;
        }

        List<BzSkuSpecSalesVo> salesBoardVoList = records.stream().map(item -> {
            BzSkuSpecSalesVo vo = new BzSkuSpecSalesVo();
            vo.setSkuSpecCode(item.getSkuSpecCode());
            vo.setSkuSpecName(item.getSkuSpecName());
            vo.setSalesQuantity(item.getSalesQuantity());
            vo.setDailySales(vo.getSalesQuantity().divide(BigDecimal.valueOf(30), 0, RoundingMode.HALF_UP));
            return vo;
        }).collect(Collectors.toList());

        result.setTotal(page.getTotal());
        result.setRecords(salesBoardVoList);
        result.setPages(page.getPages());
        result.setSize(page.getSize());
        return result;
    }
}
