package com.sd.business.service.purchase.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.apply.po.ApplyBuyBom;
import com.sd.business.entity.board.bo.OrderSalesShipmentStatisticsBo;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.purchase.dto.*;
import com.sd.business.entity.purchase.enums.PurchaseStatusEnum;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.entity.purchase.vo.PurchaseBomVo;
import com.sd.business.entity.purchase.vo.PurchaseInTransitBomVo;
import com.sd.business.entity.purchase.vo.PurchasePlanExportVo;
import com.sd.business.entity.purchase.vo.PurchaseVo;
import com.sd.business.entity.supplier.po.Supplier;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.purchase.PurchaseMapper;
import com.sd.business.service.apply.ApplyBuyBomService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.OrderSalesShipmentStatisticsService;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.business.service.purchase.PurchaseService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.supplier.SupplierService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import com.sd.framework.util.excel.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 采购合同 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Service
public class PurchaseServiceImpl extends ServiceImpl<PurchaseMapper, Purchase> implements PurchaseService {

    @Autowired
    private PurchaseBomService purchaseBomService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private ApplyBuyBomService applyBuyBomService;

    @Autowired
    private SupplierService supplierService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private OrderSalesShipmentStatisticsService orderSalesShipmentStatisticsService;

    @Autowired
    private HttpServletResponse response;

    @Override
    public Page<PurchaseVo> getPage(PurchaseSelectDto dto) {
        IWrapper<Purchase> wrapper = getWrapper();
        wrapper.eq("p", Purchase::getFlowStatus, dto.getFlowStatus());
        wrapper.eq("p", Purchase::getStatus, dto.getStatus());
        wrapper.like("s", Supplier::getName, dto.getSupplierName());
        if (StrUtil.isNotBlank(dto.getCode())) {
            wrapper.and(q -> q.like("p", Purchase::getCode, dto.getCode()).or().like("p", Purchase::getErpCode, dto.getCode()));
        }
        wrapper.orderByDesc("p", Purchase::getId);

        // 查询采购单数据
        Page<PurchaseVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        List<PurchaseVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        // 赋值采购明细
        List<Long> idList = records.stream().map(PurchaseVo::getId).collect(Collectors.toList());
        List<PurchaseBom> list = purchaseBomService.list(q -> q.in(PurchaseBom::getPurchaseId, idList));
        List<PurchaseBomVo> purchaseBomList = BeanUtil.copyToList(list, PurchaseBomVo.class);
        bomSpecService.attributeAssign(purchaseBomList, PurchaseBom::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
            item.setBomSpecColour(bomSpec.getColour());
            item.setBomSpecLength(bomSpec.getLength());
            item.setBomSpecWidth(bomSpec.getWidth());
            item.setBomSpecHeight(bomSpec.getHeight());
        });

        Map<Long, List<PurchaseBomVo>> map = purchaseBomList.stream().collect(Collectors.groupingBy(PurchaseBom::getPurchaseId));
        for (PurchaseVo record : records) {
            record.setPurchaseBomList(map.getOrDefault(record.getId(), Collections.emptyList()));
        }

        return page;
    }

    @Override
    public PurchaseVo detail(Long id) {
        Purchase Purchase = this.getById(id);
        PurchaseVo result = BeanUtil.toBean(Purchase, PurchaseVo.class);

        List<PurchaseBom> list = purchaseBomService.list(q -> q.eq(PurchaseBom::getPurchaseId, id));
        List<PurchaseBomVo> purchaseBomList = BeanUtil.copyToList(list, PurchaseBomVo.class);
        bomSpecService.attributeAssign(purchaseBomList, PurchaseBom::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
            item.setBomSpecColour(bomSpec.getColour());
            item.setBomSpecLength(bomSpec.getLength());
            item.setBomSpecWidth(bomSpec.getWidth());
            item.setBomSpecHeight(bomSpec.getHeight());
        });
        // 赋值在途数量
        purchaseBomList.forEach(item -> item.setInTransitQuantity(
                item.getPurchaseQuantity()
                .subtract(item.getArrivalQuantity())
                .add(item.getReturnQuantity())));

        result.setPurchaseBomList(purchaseBomList);
        return result;
    }

    @DSTransactional
    @Override
    public void add(PurchaseDto purchaseDto) {

        // 保存采购合同
        purchaseDto.setCode(CodeEnum.PURCHASE_CODE.getCode());
        purchaseDto.setReturnAmount(BigDecimal.ZERO);
        purchaseDto.setClosedAccountAmount(BigDecimal.ZERO);
        purchaseDto.setDeductibleAmount(BigDecimal.ZERO);
        purchaseDto.setPaymentStatus(StatusConstant.NO);
        purchaseDto.setStorageStatus(StatusConstant.NO);
        this.save(purchaseDto);

        // 保存采购明细
        Map<Long, ApplyBuyBom> applyBuyBomMap = applyBuyBomService.mapKEntity(
                BaseIdPo::getId,
                q -> q.eq(ApplyBuyBom::getApplyBuyId, purchaseDto.getApplyBuyId()));
        List<PurchaseBom> purchaseBomList = purchaseDto.getPurchaseBomList();
        purchaseBomList.forEach(item -> {
            ApplyBuyBom applyBuyBom = applyBuyBomMap.get(item.getApplyBuyBomId());
            if (applyBuyBom == null) {
                throw new ServiceException("存在未知申购明细id");
            }

            BigDecimal quantity = applyBuyBom.getQuantity();
            BigDecimal purchaseQuantity = applyBuyBom.getPurchaseQuantity();
            BigDecimal frozenQuantity = applyBuyBom.getFrozenQuantity();
            BigDecimal itemPurchaseQuantity = item.getPurchaseQuantity();

            if (quantity.compareTo(purchaseQuantity.add(frozenQuantity).add(itemPurchaseQuantity)) >= 0) {
                applyBuyBom.setFrozenQuantity(frozenQuantity.add(itemPurchaseQuantity));
            } else {
                throw new ServiceException("采购数量超过申购数量");
            }

            item.setPurchaseId(purchaseDto.getId());
            item.setReturnQuantity(BigDecimal.ZERO);
            item.setPaidAmount(BigDecimal.ZERO);
        });
        purchaseBomService.saveBatch(purchaseBomList);

        // 更新申购明细
        ArrayList<ApplyBuyBom> applyBuyBomList = new ArrayList<>(applyBuyBomMap.values());
        applyBuyBomService.updateBatchById(applyBuyBomList);

        ObsFileUtil.saveFile(purchaseDto.getFileList(), purchaseDto.getId());
    }

    @DSTransactional
    @Override
    public void edit(PurchaseDto purchaseDto) {

        purchaseDto.setCode(null);

        // 保存采购合同
        this.updateById(purchaseDto);

        Map<Long, ApplyBuyBom> applyBuyBomMap = applyBuyBomService.mapKEntity(
                BaseIdPo::getId,
                q -> q.eq(ApplyBuyBom::getApplyBuyId, purchaseDto.getApplyBuyId()));

        Map<Long, PurchaseBom> purchaseBomMap = purchaseBomService.mapKEntity(
                BaseIdPo::getId,
                q -> q.eq(PurchaseBom::getPurchaseId, purchaseDto.getId()));

        List<PurchaseBom> purchaseBomList = purchaseDto.getPurchaseBomList();
        purchaseBomList.forEach(item -> {
            item.setPurchaseId(purchaseDto.getId());

            ApplyBuyBom applyBuyBom = applyBuyBomMap.get(item.getApplyBuyBomId());
            if (applyBuyBom == null) {
                throw new ServiceException("存在未知申购明细id");
            }

            BigDecimal quantity = applyBuyBom.getQuantity();
            BigDecimal purchaseQuantity = applyBuyBom.getPurchaseQuantity();
            BigDecimal frozenQuantity = applyBuyBom.getFrozenQuantity();
            BigDecimal itemPurchaseQuantity = item.getPurchaseQuantity();

            if (item.getId() != null) {
                PurchaseBom purchaseBom = purchaseBomMap.get(item.getId());
                if (purchaseBom == null) {
                    throw new ServiceException("存在未知采购明细id");
                }
                frozenQuantity = frozenQuantity.subtract(purchaseBom.getPurchaseQuantity());
            }

            if (quantity.compareTo(purchaseQuantity.add(frozenQuantity).add(itemPurchaseQuantity)) >= 0) {
                applyBuyBom.setFrozenQuantity(frozenQuantity.add(itemPurchaseQuantity));
            } else {
                throw new ServiceException("采购数量超过申购数量");
            }

        });
        purchaseBomService.editLinked(purchaseBomList, PurchaseBom::getPurchaseId, purchaseDto.getId());

        ObsFileUtil.editFile(purchaseDto.getFileList(), purchaseDto.getId());
    }

    @DSTransactional
    @Override
    public void delete(Long id) {
        this.removeById(id);

        Map<Long, PurchaseBom> purchaseBomMap = purchaseBomService.mapKEntity(
                PurchaseBom::getApplyBuyBomId,
                q -> q.eq(PurchaseBom::getPurchaseId, id));

        List<ApplyBuyBom> applyBuyBomList = applyBuyBomService.list(q -> q.eq(ApplyBuyBom::getApplyBuyId, id));
        applyBuyBomList.forEach(item -> {
            PurchaseBom purchaseBom = purchaseBomMap.get(item.getId());
            if (purchaseBom == null) {
                return;
            }
            item.setFrozenQuantity(item.getFrozenQuantity().subtract(purchaseBom.getPurchaseQuantity()));
        });
        applyBuyBomService.updateBatchById(applyBuyBomList);

        purchaseBomService.remove(q -> q.eq(PurchaseBom::getPurchaseId, id));

        ObsFileUtil.removeFile(id);
    }

    @Override
    public List<PurchaseVo> getPurchaseInStorageList() {
        IWrapper<Purchase> wrapper = getWrapper();
        wrapper.eq("p", Purchase::getFlowStatus, FlowStatusEnum.PASS.getKey());
        wrapper.eq("p", Purchase::getStorageStatus, StatusConstant.NO);
        wrapper.eq("p", Purchase::getStatus, PurchaseStatusEnum.UNDER_PURCHASE.getKey());
        wrapper.orderByDesc(Purchase::getId);
        return this.baseMapper.getPurchaseInStorageList(wrapper);
    }

    @Override
    public void updatePurchaseBomArrivalQuantity(Long purchaseId, List<InOutStorageBom> inOutStorageBomList) {
        for (InOutStorageBom inOutStorageBom : inOutStorageBomList) {
            PurchaseBom bom = purchaseBomService.getOne(q -> q
                    .eq(PurchaseBom::getPurchaseId, purchaseId)
                    .eq(PurchaseBom::getBomSpecId, inOutStorageBom.getBomSpecId()));
            if (ObjectUtil.isNotEmpty(bom)) {
                purchaseBomService.update(q -> q
                        .eq(PurchaseBom::getId, bom.getId())
                        .set(PurchaseBom::getArrivalQuantity, bom.getArrivalQuantity().add(inOutStorageBom.getQuantity())));

                // 查询当前采购在途数量
                long count = purchaseBomService.count(q -> q
                        .eq(PurchaseBom::getPurchaseId, purchaseId)
                        .apply("(purchase_quantity + return_quantity) > arrival_quantity"));
                if (count == 0) {
                    // 如果为0，将入库状态设置为已完成
                    this.update(q -> q.set(Purchase::getStorageStatus, StatusConstant.YES)
                            .set(Purchase::getStatus, PurchaseStatusEnum.HAVE_PURCHASE.getKey())
                            .eq(Purchase::getId, purchaseId));
                }
            }
        }
    }

    @Override
    public void purchaseTermination(Long id) {
        Purchase purchase = this.getById(id);
        if (purchase == null) {
            return;
        }
        purchase.setStatus(PurchaseStatusEnum.TERMINATION.getKey());
        purchase.setStorageStatus(StatusConstant.YES);
        this.updateById(purchase);

        // 合同终止将bom到货数量设置为采购数量
        List<PurchaseBom> purchaseBomList = purchaseBomService.list(q -> q
                .eq(PurchaseBom::getPurchaseId, id)
                .apply("(purchase_quantity + return_quantity) > arrival_quantity"));
        for (PurchaseBom purchaseBom : purchaseBomList) {
            purchaseBom.setArrivalQuantity(purchaseBom.getPurchaseQuantity().add(purchaseBom.getReturnQuantity()));
        }
        purchaseBomService.updateBatchById(purchaseBomList);
    }

    @Override
    public Page<PurchaseVo> getPurchaseSelectList(PurchaseSelectListDto dto) {
        IWrapper<Purchase> wrapper = getWrapper();
        wrapper.like("p", Purchase::getCode, dto.getCode());
        wrapper.eq("p", Purchase::getFlowStatus, dto.getFlowStatus());
        wrapper.like("s", Supplier::getName, dto.getSupplierName());
        // 采购入库查询排除胜德塑胶供应商
        if (ObjectUtil.equals(dto.getQueryType(), 1)) {
            wrapper.ne("s", Supplier::getName, "福清市胜德塑胶制品有限公司");
            wrapper.eq("p", Purchase::getStatus, PurchaseStatusEnum.UNDER_PURCHASE.getKey());
        }
        wrapper.orderByDesc("p", Purchase::getId);

        // 查询采购单数据
        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public void purchaseImport(List<PurchaseBomImportDataDto> list) {
        // 验证导入数据
        validatedImportData(list);

        Map<String, List<PurchaseBomImportDataDto>> bomMap = list.stream().collect(Collectors.groupingBy(PurchaseBomImportDataDto::getPurchaseCode));
        Map<String, Long> purchaseMap = this.mapKV(Purchase::getErpCode, BaseIdPo::getId, q -> q.in(Purchase::getErpCode, bomMap.keySet()));

        Set<String> supplierCodes = list.stream().map(PurchaseBomImportDataDto::getSupplierCode).collect(Collectors.toSet());
        Map<String, Long> supplierMap = supplierService.mapKV(Supplier::getCode, BaseIdPo::getId, q -> q.in(Supplier::getCode, supplierCodes));

        List<String> bomSpecCodes = list.stream().map(PurchaseBomImportDataDto::getBomSpecCode).collect(Collectors.toList());
        Map<String, BomSpec> bomSpecMap = bomSpecService.mapKEntity(BomSpec::getCode, q -> q.in(BomSpec::getCode, bomSpecCodes));
        List<Long> bomSpecIds = bomSpecMap.values().stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);

        List<Purchase> purchaseList = new ArrayList<>();
        List<PurchaseBom> purchaseBomList = new ArrayList<>();
        List<BomSpec> bomSpecList = new ArrayList<>();
        for (Map.Entry<String, List<PurchaseBomImportDataDto>> entry : bomMap.entrySet()) {
            String purchaseCode = entry.getKey();
            List<PurchaseBomImportDataDto> bomList = entry.getValue();
            if (ObjectUtil.isEmpty(bomList)) {
                continue;
            }

            // 判断是否存在重复编号
            Long purchaseId = purchaseMap.get(purchaseCode);
            if (purchaseId != null) {
                throw new ServiceException("采购合同编号：" + purchaseCode + " 已存在");
            }

            Long supplierId = supplierMap.get(bomList.get(0).getSupplierCode());
            if (supplierId == null) {
                throw new ServiceException("未知供应商编号：" + bomList.get(0).getSupplierCode());
            }

            // 保存采购合同
            Purchase purchase = new Purchase();
            purchase.setId(IdWorker.getId());
            purchase.setCode(CodeEnum.PURCHASE_CODE.getCode());
            purchase.setSupplierId(supplierId);
            purchase.setStatus(PurchaseStatusEnum.UNDER_PURCHASE.getKey());
            purchase.setFlowStatus(FlowStatusEnum.PASS.getKey());
            purchase.setErpCode(purchaseCode);
            purchase.setReceiveGoodsType(1);
            purchase.setAdvancePayment(BigDecimal.ZERO);
            purchase.setReturnAmount(BigDecimal.ZERO);
            purchase.setClosedAccountAmount(BigDecimal.ZERO);
            purchase.setDeductibleAmount(BigDecimal.ZERO);
            purchase.setPaymentStatus(StatusConstant.NO);
            purchase.setStorageStatus(StatusConstant.NO);
            purchase.setTotalAmountIncludingTax(BigDecimal.ZERO);
            purchase.setTotalAmountExcludingTax(BigDecimal.ZERO);
            purchase.setDeliveryDate(bomList.get(0).getDeliveryDate());


            List<PurchaseBom> bomData = bomList.stream().map(item -> {
                BomSpec bomSpec = bomSpecMap.get(item.getBomSpecCode());
                if (bomSpec == null) {
                    throw new ServiceException("未知bom品号：" + item.getBomSpecCode());
                }
                PurchaseBom purchaseBom = new PurchaseBom();
                purchaseBom.setPurchaseId(purchase.getId());
                purchaseBom.setBomSpecId(bomSpec.getId());
                purchaseBom.setUnitPrice(new BigDecimal(item.getUnitPrice()));
                purchaseBom.setTaxRate(BigDecimal.TEN);
                purchaseBom.setPurchaseQuantity(new BigDecimal(item.getQuantity()));
                purchaseBom.setArrivalQuantity(BigDecimal.ZERO);
                purchaseBom.setReturnQuantity(BigDecimal.ZERO);
                purchaseBom.setPaidAmount(BigDecimal.ZERO);

                purchase.setTotalAmountIncludingTax(purchase.getTotalAmountIncludingTax().add(new BigDecimal(item.getTotalAmount())));
                // 计算不含税总金额
                BigDecimal totalAmountIncludingTax = purchaseBom.getUnitPrice()
                        .divide(new BigDecimal("1.1"), 4, RoundingMode.HALF_UP)
                        .multiply(purchaseBom.getPurchaseQuantity());
                purchase.setTotalAmountExcludingTax(purchase.getTotalAmountExcludingTax().add(totalAmountIncludingTax));

                // 主材保存最新的合同价格
                if (Objects.equals(bomSpecBoMap.get(bomSpec.getId()).getClassifyParentId(), 1L)) {
                    bomSpec.setCostPrice(purchaseBom.getUnitPrice());
                    bomSpec.setInternalSellingPrice(purchaseBom.getUnitPrice());
                    bomSpecList.add(bomSpec);
                }

                return purchaseBom;
            }).collect(Collectors.toList());
            purchaseList.add(purchase);
            purchaseBomList.addAll(bomData);
            purchaseMap.put(purchaseCode, purchase.getId());
        }

        this.saveBatch(purchaseList);
        purchaseBomService.saveBatch(purchaseBomList);
        if (!bomSpecList.isEmpty()) {
            bomSpecService.updateBatchById(bomSpecList);
        }
    }

    @Override
    public void purchasePlanExportExcel(List<PurchasePlanImportDataDto> list) {
        // 验证采购计划导入数据
        validatedPurchasePlanImportData(list);

        List<String> bomSpecCodes = list.stream().map(PurchasePlanImportDataDto::getBomSpecCode).collect(Collectors.toList());
        Map<String, BomSpec> bomSpecMap = bomSpecService.mapKEntity(BomSpec::getCode, q -> q.in(BomSpec::getCode, bomSpecCodes));
        List<Long> bomSpecIds = bomSpecMap.values().stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 查询库存数据
        List<Inventory> inventorList = inventoryService.list(q -> q
                .in(Inventory::getBomSpecId, bomSpecIds)
                .eq(Inventory::getDepartmentId, DepartmentConstant.SD_SPORTS)
                .eq(Inventory::getWarehouseId, WarehouseConstant.SEMI_FINISHED_PRODUCT));
        Map<Long, BigDecimal> inventorMap = inventorList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, item -> item.getQuantity().add(item.getLockQuantity())));

        // 获取在途数量
        List<PurchaseInTransitBomVo> purchaseBomInTransitList = purchaseBomService.getPurchaseBomInTransitSum(bomSpecIds);
        Map<Long, BigDecimal> inTransitMap = purchaseBomInTransitList.stream().collect(Collectors.toMap(PurchaseInTransitBomVo::getBomSpecId, PurchaseInTransitBomVo::getInTransitQuantity));

        // 获取销售量
        Map<Long, OrderSalesShipmentStatisticsBo> shipmentStatisticsBoMap = orderSalesShipmentStatisticsService.getSalesShipmentStatisticsByDate(bomSpecIds);

        List<PurchasePlanExportVo> exportDataList = new ArrayList<>();

        for (PurchasePlanImportDataDto importDataDto : list) {
            PurchasePlanExportVo vo = new PurchasePlanExportVo();
            BomSpec bomSpec = bomSpecMap.get(importDataDto.getBomSpecCode());
            if (bomSpec == null) {
                throw new ServiceException("未知bom品号：" + importDataDto.getBomSpecCode());
            }
            Long bomSpecId = bomSpec.getId();
            vo.setStockQuantity(inventorMap.getOrDefault(bomSpecId, BigDecimal.ZERO));
            vo.setInTransitQuantity(inTransitMap.getOrDefault(bomSpecId, BigDecimal.ZERO));
            vo.setActualDailySales(BigDecimal.ZERO);
            OrderSalesShipmentStatisticsBo shipmentStatisticsBo = shipmentStatisticsBoMap.get(bomSpecId);
            if (shipmentStatisticsBo != null) {
                vo.setActualDailySales(shipmentStatisticsBo.getThirtyDaysSalesQuantity().divide(BigDecimal.valueOf(30), 0, RoundingMode.HALF_UP));
            }

            // 规格
            String length = bomSpec.getLength() == null ? "0" : bomSpec.getLength().stripTrailingZeros().toPlainString();
            String width = bomSpec.getWidth() == null ? "0" : bomSpec.getWidth().stripTrailingZeros().toPlainString();
            String height = bomSpec.getHeight() == null ? "0" : bomSpec.getHeight().stripTrailingZeros().toPlainString();
            // 赋值
            vo.setBomSpecCode(bomSpec.getCode());
            vo.setBomSpecName(bomSpec.getName());
            vo.setSpecification(length + " * " + width + " * " + height);
            vo.setColour(bomSpec.getColour());
            vo.setUnit("PCS");
            vo.setStockingMode("预测备货");
            vo.setMinimumQuantity(BigDecimal.valueOf(300));
            vo.setPurchaseCycle(BigDecimal.valueOf(40));
            vo.setSafetyStock(vo.getActualDailySales().multiply(vo.getPurchaseCycle()));
            vo.setForecastDemandQuantity(importDataDto.getForecastDemandQuantity());
            // 预测库销天数 = (库存数量+在途数量)/(预测需求/60)
            vo.setForecastInventorySalesDays(
                    vo.getStockQuantity().add(vo.getInTransitQuantity())
                            .divide(vo.getForecastDemandQuantity().divide(BigDecimal.valueOf(60), 4, RoundingMode.HALF_UP), 0, RoundingMode.DOWN));
            // 实际库销天数 = (库存数量+在途数量)/实际日销
            if (vo.getActualDailySales().compareTo(BigDecimal.ZERO) == 0) {
                vo.setActualInventorySalesDays(BigDecimal.ZERO);
            } else {
                vo.setActualInventorySalesDays(
                        vo.getStockQuantity().add(vo.getInTransitQuantity())
                                .divide(vo.getActualDailySales(), 0, RoundingMode.DOWN));
            }
            // 短缺数量合计(销售预测） = 预测需求-库存数量-在途数量+安全库存
            vo.setForecastShortageQuantity(
                    vo.getForecastDemandQuantity()
                            .subtract(vo.getStockQuantity())
                            .subtract(vo.getInTransitQuantity())
                            .add(vo.getSafetyStock()));
            // 短缺数量合计（实际销量） = 实际日销*60-库存数量-在途数量+安全库存
            BigDecimal actualShortageQuantity = vo.getActualDailySales()
                    .multiply(BigDecimal.valueOf(60))
                    .subtract(vo.getStockQuantity())
                    .subtract(vo.getInTransitQuantity())
                    .add(vo.getSafetyStock());
            vo.setActualShortageQuantity(actualShortageQuantity.compareTo(BigDecimal.ZERO) < 0 ? BigDecimal.ZERO : actualShortageQuantity);
            // 经济采购批量 = 短缺数量合计(销售预测）/60*采购周期
            if (vo.getForecastShortageQuantity().compareTo(BigDecimal.ZERO) < 0) {
                vo.setEconomicOrderQuantity(BigDecimal.ZERO);
            } else {
                vo.setEconomicOrderQuantity(
                        vo.getForecastShortageQuantity()
                                .divide(BigDecimal.valueOf(60), 4, RoundingMode.HALF_UP)
                                .multiply(vo.getPurchaseCycle())
                                .setScale(0, RoundingMode.HALF_UP));
            }
            // 采购批次 = 短缺数量合计(销售预测）/经济采购批量
            if (vo.getEconomicOrderQuantity().compareTo(BigDecimal.ZERO) == 0) {
                vo.setPurchaseBatches(BigDecimal.ZERO);
            } else {
                vo.setPurchaseBatches(
                        vo.getForecastShortageQuantity()
                                .divide(vo.getEconomicOrderQuantity(), 0, RoundingMode.UP));
            }

            exportDataList.add(vo);
        }

        ExcelUtil.export(response,"胜德体育滚动采购计划", "胜德体育采购计划", exportDataList, PurchasePlanExportVo.class);
    }

    private void validatedPurchasePlanImportData(List<PurchasePlanImportDataDto> list) {
        Assert.notEmpty(list, "采购合同明细不能为空");
        for (PurchasePlanImportDataDto dto : list) {
            Assert.notBlank(dto.getBomSpecCode(), "bom品号不能为空");
            Assert.notNull(dto.getForecastDemandQuantity(), "预测需求格式不正确");
        }
    }

    private void validatedImportData(List<PurchaseBomImportDataDto> list) {
        Assert.notEmpty(list, "采购合同明细不能为空");
        for (PurchaseBomImportDataDto dto : list) {
            Assert.notBlank(dto.getPurchaseCode(), "采购合同编号不能为空");
            Assert.notBlank(dto.getSupplierCode(), "供应商编号不能为空");
            Assert.notBlank(dto.getBomSpecCode(), "bom品号不能为空");
            Assert.eqTrue(NumberUtil.isNumber(dto.getQuantity()), "采购数量格式不正确");
            Assert.eqTrue(NumberUtil.isNumber(dto.getUnitPrice()), "采购单价格式不正确");
            Assert.eqTrue(NumberUtil.isNumber(dto.getTotalAmount()), "采购金额合计格式不正确");
            Assert.gtZero(new BigDecimal(dto.getQuantity()), "采购数量必须大于0");
            Assert.gtZero(new BigDecimal(dto.getUnitPrice()), "采购单价必须大于0");
        }
    }

}
