package com.sd.wln.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.constant.BomClassifyConstant;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.service.order.*;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.util.CodeEnum;
import com.sd.wln.service.WlnSalesReturnStockInService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WLnSalesReturnStockInServiceImpl implements WlnSalesReturnStockInService {


    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private OrderExchangeService orderExchangeService;

    @Autowired
    private OrderExchangeDetailService orderExchangeDetailService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void syncSalesReturnStockIn() {
        // 获取万里牛销售退货入库明细
        List<JSONObject> data = getWlnSalesReturnStockIn();

        List<JSONObject> wlnSalesReturnStockInList = data.stream()
                .filter(item -> Objects.equals(item.getString("storage_code"), "T007"))
                .collect(Collectors.toList());

        if (ObjectUtil.isEmpty(wlnSalesReturnStockInList)) {
            return;
        }

        // 获取订单数据
        Map<Integer, List<String>> orderCodeMap = wlnSalesReturnStockInList.stream()
                .map(item -> item.getString("tp_tid"))
                .collect(Collectors.groupingBy(item -> item.startsWith("XD") ? 1 : 0));
        List<String> orderCodeList = orderCodeMap.getOrDefault(0, Collections.emptyList());
        List<String> orderWlnCodeList = orderCodeMap.getOrDefault(1, Collections.emptyList());

        List<OrderInfo> orderInfoList = orderService.list(q -> q.eq(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
                .and(i -> i.in(!orderCodeList.isEmpty(), OrderInfo::getCode, orderCodeList)
                        .or(!orderWlnCodeList.isEmpty(), o -> o.in(OrderInfo::getWlnCode, orderWlnCodeList))));

        List<Long> orderIds = orderInfoList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<OrderSku>> orderSkuMap = orderSkuService.mapKGroup(OrderSku::getOrderId, q -> q.in(OrderSku::getOrderId, orderIds));
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomService.mapKGroup(OrderSkuBom::getOrderSkuId, q -> q.in(OrderSkuBom::getOrderId, orderIds));
        List<Long> bomSpecIdList = orderSkuBomMap.values().stream()
                .flatMap(item -> item.stream().map(OrderSkuBom::getBomSpecId))
                .distinct()
                .collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);

        // 获取sku信息
        List<String> skuSpecList = wlnSalesReturnStockInList.stream()
                .map(item -> item.getJSONArray("details").toJavaList(JSONObject.class))
                .flatMap(Collection::stream)
                .map(item -> item.getString("sku_no"))
                .distinct()
                .collect(Collectors.toList());
        Map<String, SkuSpec> skuSpecMap = skuSpecService.mapKEntity(SkuSpec::getCode, q -> q.in(SkuSpec::getCode, skuSpecList));

        // 订单单号可能会有多个
        Map<String, List<OrderInfo>> codeOrderMap = orderInfoList.stream().collect(Collectors.groupingBy(OrderInfo::getCode));
        Map<String, OrderInfo> wlnCodeOrderMap = orderInfoList.stream().collect(Collectors.toMap(OrderInfo::getWlnCode, Function.identity()));

        List<OrderExchange> orderExchangeList = new ArrayList<>();
        List<OrderExchangeDetail> orderExchangeDetailList = new ArrayList<>();

        String thCode = CodeEnum.TH_CODE.getCode();
        for (JSONObject item : wlnSalesReturnStockInList) {
            // 获取订单，订单号存在两种，XD和普通单号
            String orderCode = item.getString("tp_tid");
            List<OrderInfo> orderList = new ArrayList<>();
            if (orderCode.startsWith("XD")) {
                OrderInfo orderInfo = wlnCodeOrderMap.get(orderCode);
                if (orderInfo != null) {
                    orderList.add(orderInfo);
                }
            } else {
                orderList = codeOrderMap.getOrDefault(orderCode, Collections.emptyList());
            }

            if (orderList.isEmpty()) {
                continue;
            }

            Map<Long, OrderInfo> orderInfoMap = orderList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

            // 获取订单明细
            List<OrderSku> orderSkuList = orderList.stream()
                    .map(i -> orderSkuMap.getOrDefault(i.getId(), Collections.emptyList()))
                    .flatMap(Collection::stream)
                    .collect(Collectors.toList());

            // 入库单明细
            List<JSONObject> detailsList = item.getJSONArray("details").toJavaList(JSONObject.class);

            // 出库单明细
            OrderExchange orderExchange = null;

            for (JSONObject itemDetails : detailsList) {
                // 获取sku信息
                SkuSpec skuSpec = skuSpecMap.get(itemDetails.getString("sku_no"));
                if (skuSpec == null || Objects.equals(skuSpec.getGiftTag(), StatusConstant.YES)) {
                    continue;
                }

                // 通过sku获取订单明细
                OrderSku orderSku = orderSkuList.stream()
                        .filter(i -> Objects.equals(i.getSkuSpecId(), skuSpec.getId()))
                        .findFirst()
                        .orElse(null);
                if (orderSku == null) {
                    continue;
                }

                OrderInfo orderInfo = orderInfoMap.get(orderSku.getOrderId());
                // 无理由订单不处理
                if (orderInfo == null || Objects.equals(orderInfo.getClassify(), OrderClassifyEnum.NO_REASON_ORDER.getKey())) {
                    continue;
                }

                // 包材成本只退可二次利用的包材
                List<OrderSkuBom> orderSkuBomList = orderSkuBomMap.getOrDefault(orderSku.getId(), Collections.emptyList());
                BigDecimal packagingMaterialCost = orderSkuBomList.stream()
                        .map(b -> {
                            BomSpecBo bomSpecBo = bomSpecBoMap.get(b.getBomSpecId());
                            if (bomSpecBo == null
                                    || !Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.MESH_BAG)) {
                                return BigDecimal.ZERO;
                            }
                            return b.getUnitPrice().multiply(b.getQuantity());
                        })
                        .reduce(BigDecimal.ZERO, BigDecimal::add);

                // 新增售后单数据
                if (orderExchange == null) {
                    orderExchange = new OrderExchange();
                    orderExchange.setId(IdWorker.getId());
                    orderExchange.setOrderInfoId(orderInfo.getId());
                    orderExchange.setCode(thCode);
                    orderExchange.setWarehouseId(WarehouseConstant.FINISHED_PRODUCT);
                    orderExchange.setType(StatusConstant.YES);
                    orderExchange.setStatus(StatusConstant.NO);
                    orderExchange.setReason(item.getString("reason_name"));
                    orderExchange.setRemark(item.getString("sys_remark"));
                    orderExchange.setCompletionTime(item.getDate("modify_time"));
                    orderExchange.setCompletionTimestamp(item.getLong("modify_time"));
                    orderExchangeList.add(orderExchange);
                    // 递增售后单号
                    Integer codeNumber = Convert.toInt(thCode.split("-")[2]);
                    thCode = thCode.substring(0, thCode.lastIndexOf("-") + 1) + String.format("%06d", codeNumber + 1);
                }
                OrderExchangeDetail orderExchangeDetail = new OrderExchangeDetail();
                orderExchangeDetail.setOrderExchangeId(orderExchange.getId());
                orderExchangeDetail.setQuantity(itemDetails.getBigDecimal("nums"));
                orderExchangeDetail.setReturnStatus(StatusConstant.NO);
                orderExchangeDetail.setExchangeStatus(StatusConstant.NO);
                orderExchangeDetail.setOrderSkuId(orderSku.getId());
                orderExchangeDetail.setCheckPassesQuantity(BigDecimal.ZERO);
                orderExchangeDetail.setMeshBagDamageQuantity(BigDecimal.ZERO);
                orderExchangeDetail.setReturnAmount((orderSku.getUnitPrice()
                        .add(packagingMaterialCost)).multiply(orderExchangeDetail.getQuantity()));
                orderExchangeDetailList.add(orderExchangeDetail);
            }
        }

        // 新增售后订单
        orderExchangeService.saveBatch(orderExchangeList);
        orderExchangeDetailService.saveBatch(orderExchangeDetailList);
    }

    /**
     * 查询万里牛销售退货入库单数据
     */
    private List<JSONObject> getWlnSalesReturnStockIn() {
        OrderExchange orderExchange = orderExchangeService.getOne(q -> q
                .isNotNull(OrderExchange::getCompletionTimestamp)
                .orderByDesc(OrderExchange::getCompletionTimestamp));

        long startTime;
        long endTime = System.currentTimeMillis();
        if (orderExchange != null) {
            startTime = orderExchange.getCompletionTimestamp() + 1;
        } else {
            startTime = endTime - 1000 * 60 * 60 * 24 * 7;
        }

        List<JSONObject> list = new ArrayList<>();

        int page = 1;
        int size;
        do {
            try {
                List<JSONObject> itemList = WlnUtil.getSalesReturnStockInList(page, 200, startTime, endTime);
                page++;
                size = itemList.size();
                list.addAll(itemList);
            } catch (Exception e) {
                log.error("销售退货入库单同步失败", e);
                return Collections.emptyList();
            }
        } while (size >= 200);

        return list;
    }
}
