package com.sd.wln.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.StatusConstant;
import com.sd.business.entity.after.po.AfterSaleOrder;
import com.sd.business.service.after.AfterSaleOrderService;
import com.sd.wln.service.WlnReturnOrderService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WlnReturnOrderServiceImpl implements WlnReturnOrderService {

    @Autowired
    private AfterSaleOrderService afterSaleOrderService;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void syncReturnOrder() {
        // 获取万里牛数据
        List<JSONObject> wlnReturnOrderList = getWlnReturnOrder();
        // 筛选条件
        List<Integer> wlnTypeList = Arrays.asList(0, 1, 4);
        List<Integer> wlnStatusList = Arrays.asList(0, 1);

        if (ObjectUtil.isEmpty(wlnReturnOrderList)) {
            return;
        }

        // 查询万里牛售后单明细id
        List<String> uidList = wlnReturnOrderList.stream()
                .map(item -> item.getJSONArray("items").toJavaList(JSONObject.class))
                .flatMap(Collection::stream)
                .map(item -> item.getString("tp_off_id"))
                .collect(Collectors.toList());
        Map<String, AfterSaleOrder> afterSaleOrderMap = afterSaleOrderService.mapKEntity(AfterSaleOrder::getWlnOffId, q -> q.in(AfterSaleOrder::getWlnOffId, uidList));

        List<AfterSaleOrder> saveAfterSaleOrderList = new ArrayList<>();
        List<AfterSaleOrder> editAfterSaleOrderList = new ArrayList<>();
        List<AfterSaleOrder> removeAfterSaleOrderList = new ArrayList<>();

        for (JSONObject item : wlnReturnOrderList) {
            // 同步类型为退货和仅退款的订单
            if (!wlnTypeList.contains(item.getInteger("type"))) {
                continue;
            }

            // 出库单明细
            List<JSONObject> detailsList = item.getJSONArray("items").toJavaList(JSONObject.class);
            for (JSONObject itemDetails : detailsList) {

                AfterSaleOrder afterSaleOrder = afterSaleOrderMap.get(itemDetails.getString("tp_off_id"));

                // 新增
                if (afterSaleOrder == null) {
                    // 新增只同步状态为处理中和已完成的售后单
                    if (!wlnStatusList.contains(item.getInteger("status"))) {
                        continue;
                    }

                    afterSaleOrder = new AfterSaleOrder();
                    afterSaleOrder.setCode(item.getString("bill_code"));
                    afterSaleOrder.setStatus(StatusConstant.NO);
                    afterSaleOrder.setWlnOffId(itemDetails.getString("tp_off_id"));
                    afterSaleOrder.setOrderCode(item.getString("oln_trade_code"));
                    afterSaleOrder.setOrderWlnCode(item.getString("trade_code"));
                    afterSaleOrder.setSkuSpecCode(itemDetails.getString("sku_code"));
                    afterSaleOrder.setWlnSkuSpecId(itemDetails.getString("sys_spec_uid"));
                    afterSaleOrder.setStorageCode(item.getString("storage_code"));
                    afterSaleOrder.setWlnStatus(item.getInteger("status"));
                    afterSaleOrder.setWlnType(item.getInteger("type"));
                    afterSaleOrder.setQuantity(itemDetails.getBigDecimal("size"));
                    afterSaleOrder.setModifyTime(item.getDate("create_time"));
                    afterSaleOrder.setModifyTimestamp(item.getLong("create_time"));
                    afterSaleOrder.setReason(item.getString("reason"));
                    afterSaleOrder.setRemark(item.getString("describe"));
                    saveAfterSaleOrderList.add(afterSaleOrder);
                } else {
                    // 已售后的不修改
                    if (Objects.equals(afterSaleOrder.getStatus(), StatusConstant.YES)) {
                        continue;
                    }

                    // 售后单状态不为处理中和已完成的售后单删除
                    if (!wlnStatusList.contains(item.getInteger("status"))) {
                        removeAfterSaleOrderList.add(afterSaleOrder);
                        continue;
                    }

                    afterSaleOrder.setWlnStatus(item.getInteger("status"));
                    afterSaleOrder.setWlnType(item.getInteger("type"));
                    afterSaleOrder.setModifyTime(item.getDate("create_time"));
                    afterSaleOrder.setModifyTimestamp(item.getLong("create_time"));
                    editAfterSaleOrderList.add(afterSaleOrder);
                }
            }
        }

        if (!saveAfterSaleOrderList.isEmpty()) {
            afterSaleOrderService.saveBatch(saveAfterSaleOrderList);
        }
        if (!editAfterSaleOrderList.isEmpty()) {
            afterSaleOrderService.updateBatchById(editAfterSaleOrderList);
        }
        if (!removeAfterSaleOrderList.isEmpty()) {
            afterSaleOrderService.removeBatchByIds(removeAfterSaleOrderList);
        }
    }

    /**
     * 查询近万里牛售后单
     */
    private List<JSONObject> getWlnReturnOrder() {
        AfterSaleOrder afterSaleOrder = afterSaleOrderService.getOne(q -> q.orderByDesc(AfterSaleOrder::getModifyTimestamp));

        long startTime;
        long endTime = System.currentTimeMillis();
        if (afterSaleOrder != null) {
            startTime = afterSaleOrder.getModifyTimestamp() + 1;
        } else {
            startTime = endTime - 1000 * 60 * 60 * 24 * 7;
        }

        List<JSONObject> list = new ArrayList<>();

        int page = 1;
        int size;
        do {
            try {
                List<JSONObject> itemList = WlnUtil.getReturnOrderList(page, 200, startTime, endTime);
                page++;
                size = itemList.size();
                list.addAll(itemList);
            } catch (Exception e) {
                log.error("售后单同步失败", e);
                return Collections.emptyList();
            }
        } while (size >= 200);

        return list;
    }

}
